({
    extendsFrom: 'BaseField',
    rowId: 0,

    formRowTemplate: null,

    hasSystemName : false,

    notifiedOfParseError: false,

    fields: [],

    initialize: function(arguments){
        this._super("initialize", [arguments]);

        this.events = _.extend(this.events, {
            'click [px-context-action=add]': 'addRow',
            'click [px-context-action=remove]': 'removeRow',
            'click [px-context-action=add_to_editor]': 'addToEditor',
        });

        this.formRowTemplate = app.template.getField('px-context', 'context-row', this.module);

        this.fields = [];
        this.notifiedOfParseError = false;

        //this.model.on("change:tpl_engine_type", function(model, value) {
            //this.hasSystemName = (value == 'smarty');
            //this._render(this.getFormattedContextValue());
            //this.check()
        //}, this);
    },

    bindDataChange: function() {
        if (this.model) {
            this.model.on("change:" + this.name, function(model, value) {
                this.render();
            }, this);
        }
    },

    _render: function(values) {
        this.rowId = 0;
        this._loadTemplate();
        this.$el.html(this.template(this) || '');

        this.renderFields(values);
        this.displayInsertButton();

    },

    displayInsertButton: function(){
        if(this.disposed || _.isEmpty(this.$el) || !this.$el.find('.px-context-actions').length)
            return;

        if(this.action == 'detail')
            this.$el.find('.px-context-actions').hide();
        else
            this.$el.find('.px-context-actions').show();
    },

    renderFields: function(values){
        var defs = values || this.model.get('context');

        if(!_.isEmpty(defs) && _.isString(defs)){
            try{
                defs = JSON.parse(defs);
            }catch(err){
                defs = null;
                var errorMsg = app.lang.get('LBL_CONTEXT_PARSE_ERROR', 'PX_FlowchartTemplates');
                app.logger.warn(errorMsg);
                if(!this.notifiedOfParseError) {
                    this.notifiedOfParseError = true;
                    app.alert.show('context_parse_error', {
                        level: 'warning',
                        messages: errorMsg
                    });
                }
            }
        }

        if(_.isEmpty(defs) || (_.isArray(defs) && defs.length <= 0) || (_.isString(defs) && defs == '[]')){
            this.renderRow(null, {});
            return;
        }

        var self = this;
        _.each(defs, function(data, key){
            self.renderRow(null, data);
        });
    },

    getRowSequenceNumber: function(data){
        if(_.isUndefined(data['key'])){
            return ++this.rowId;
        }

        if(data['key'] > this.rowId)
            this.rowId = data['key'];

        return data['key'];
    },

    addRow: function(e){
        var $row;
        if (e) {
            $row = this.$(e.currentTarget).closest('[data-container=px-context-row]');
            $row.after(this.formRowTemplate({hasSystemName: this.hasSystemName}));
            $row = $row.next();
        }

        this.renderRow($row, {});

        var rowInput = $row.find('input[type="text"]');
        if(rowInput.length)
            rowInput.focus();
    },

    removeRow: function(e){
        var $row = this.$(e.currentTarget).closest('[data-container=px-context-row]');

        this.removeModelValues($row);

        $row.remove();
        if (this.$('[data-container=px-context-row]').length === 0) {
            this.addRow();
        }
    },

    removeModelValues: function($row){
        var data = $row.data(),
            labelName = data['label'],
            keyName = data['name'];

        this.model.set(labelName, '');
        this.model.set(keyName, '');
    },

    addToEditor: function(e){
        var engineType = this.model.get('tpl_engine_type'),
            $row = this.$(e.currentTarget).closest('[data-container=px-context-row]'),
            data = $row.data(),
            key = data['key'],
            fieldName = this.getAttributeLabelName(key),
            fieldValue = this.model.get(fieldName),
            formattedText = '';

        if(!this.isTemplateVariableValid(fieldValue, engineType)){
            return;
        }
        switch(engineType){
            case 'smarty':
                formattedText = this.getSmartyTemplateVariable(fieldValue)
                break;
            default:
                formattedText = this.getBasicTemplateVariable(fieldValue);
        }
        //if (engineType == 'smarty')
        //    formattedText = this.getSmartyTemplateVariable(fieldValue)
        //else
        //    formattedText = this.getBasicTemplateVariable(fieldValue);

        if(!_.isUndefined(this.view.getField('html_body')) && !this.view.getField('html_body').disposed)
            this.view.getField('html_body')._htmleditor.execCommand('mceInsertContent', false, formattedText);
    },

    isTemplateVariableValid: function(variable, engineType){
        var errFlag = true,
            errText = '';
        if(_.isEmpty(variable)){
            errFlag = false;
            errText = app.lang.get('LBL_ERR_EMPTY_TPL_VARIABLE', this.module);
        }
        switch(engineType){
            case 'smarty':
                if(variable.match(/[^a-zA-Z0-9\_]/)){
                    errFlag = false;
                    errText = app.lang.get('LBL_ERR_INCORRECT_SYMBOLS_IN_SMARTY_TPL_VARIABLE', this.module);
                }
                break;
        }
        if(!_.isEmpty(errText)){
            app.alert.show('lb_template_err_variable', {
                level: 'error',
                title: 'Attention!',
                messages: errText
            });
        }
        return errFlag;
    },

    getSmartyTemplateVariable: function(variable){
        //var key = data['key'],
        //    fieldKeyName = this.getAttributeSystemName(key),
        //    fieldKeyValue = this.model.get(fieldKeyName);

        //if(!_.isEmpty(fieldValue))
            return '{$' + variable + '}';
        //return '';
    },

    getBasicTemplateVariable: function(variable){
        //var key = data['key'],
        //    fieldLabelName = this.getAttributeLabelName(key),
        //    fieldLabelValue = this.model.get(fieldLabelName);

        //if(!_.isEmpty(fieldLabelValue))
            return '{{' + variable + '}}';
        //return '';
    },

    renderRow: function($row, data){
        $row = $row || $(this.formRowTemplate({hasSystemName: this.hasSystemName})).appendTo(this.$el);
        data = data || {};

        var rowId = this.getRowSequenceNumber(data);

        this.renderLabelField($row, data, rowId);
        this.renderSystemName($row, data, rowId);

        $row.data('key', rowId);

    },

    renderLabelField: function($row, data, rowId){
        var field, $fieldLabel, $fieldLabelContainer,
            fieldLabelName = this.getAttributeLabelName(rowId),
            fieldLabelValue = data.label || '';

        $row.data('label', fieldLabelValue);
        this.model.set(fieldLabelName, fieldLabelValue);
        var fieldDefs = _.extend({
            name: fieldLabelName,
            type: 'px-context-item',
            disabled: true,
            layout: this
        }, this.def.options.label || {});
        field = this.createField(this.model, fieldDefs);
        $fieldLabel = $row.find('[data-container=px-arguments-label]');
        $fieldLabelContainer = $(field.getPlaceholder().string);
        $fieldLabelContainer.appendTo($fieldLabel);

        this.renderField(field, $fieldLabelContainer);
        this.fields.push(field);
    },

    renderSystemName: function($row, data, rowId){
        var fieldDefs, field,
            $fieldKey, $fieldKeyContainer,
            fieldKeyName = this.getAttributeSystemName(rowId),
            fieldKeyValue = data.name || fieldKeyName;

        if(this.hasSystemName){
            $row.data('name', fieldKeyValue);
            this.model.set(fieldKeyName, fieldKeyValue);

            fieldDefs = {
                name: fieldKeyName,
                type: 'px-context-item'
            };
            if (!_.isUndefined(this.def.options.name)) {
                fieldDefs = _.extend(fieldDefs, this.def.options.name);
            }
            field = this.createField(this.model, fieldDefs);

            $fieldKey = $row.find('[data-container=px-context-key]');

            $fieldKeyContainer = $(field.getPlaceholder().string);
            $fieldKeyContainer.appendTo($fieldKey);

            this.renderField(field, $fieldKeyContainer);
            this.fields.push(field);
        }else{
            // data.name = data.name || fieldKeyName;
            $row.data('name', fieldKeyValue);
            this.model.set(fieldKeyName, fieldKeyValue);
        }
    },

    getAttributeSystemName: function(key){
        return 'attribute_' + key
    },

    getAttributeLabelName: function(key){
        return 'attribute_label_' + key
    },

    createField: function(model, def) {
        var obj = {
            meta: {
                view: 'edit'
            },
            def: def,
            model: model,
            context: app.controller.context,
            viewName: this.action,
            view: this.view
        };

        model.fields[def.name] = def;

        var field = app.view.createField(obj);
        field.action = this.action;
        return field;
    },

    renderField: function(field, $fieldEl) {
        field.setElement($fieldEl);
        try {
            field.render();
        } catch (e) {
            app.logger.error("Failed to render " + field + " on " + this.view + "\n" + e);
            app.error.handleRenderError(this.view, '_renderField', field);
        }
    },

    toggle: function(evt) {
        this.$('[data-container=px-context-row]').html('');
        this.renderFields();
    },

    clearErrorDecoration: function () {
        _.each(this.fields, function(field, key) {
            if(_.isEmpty(field) || _.isEmpty(field.$el)){
                // delete this.fields[key];
            }else {
                field.clearErrorDecoration();
            }
        }, this);

        this._super('clearErrorDecoration');
    },

    beforeHandleSave: function(){
        this.model.set('context', this.getFormattedContextValue());
    },

    getContextFieldData: function(){
        var $rows = this.$('[data-container=px-context-row]');
        var contextData = [];

        _.each($rows, function(row) {
            var argumentItem = this.getArgument($(row));
            if(!_.isEmpty(argumentItem.name) && !_.isEmpty(argumentItem.label))
                contextData.push(argumentItem);
        }, this);

        return contextData;
    },

    getFormattedContextValue: function(){
        var contextData = this.getContextFieldData();
        return JSON.stringify(contextData, null);
    },

    getArgument: function($row) {
        var data = $row.data(),
            key = data['key'],
            fieldKeyName = this.getAttributeSystemName(key),
            fieldLabelName = this.getAttributeLabelName(key);

        return {
            label: this.model.get(fieldLabelName),
            name: this.model.get(fieldKeyName),
            key: key
        };
    },

    _dispose: function() {
        _.each(this.fields, function(field){
            field.dispose();
        });
        this.fields = [];
        this._super('_dispose');
    },

    isFieldLengthValid: function(){
        if(_.isEmpty(this.model.get('context')))
            return true;

        var contextField = btoa(escape(this.model.get('context')));
        if(contextField.length >= 65535){
            return false;
        }
        return true;
    },

    getSymbolsAmountViolation: function(){
        var contextField = btoa(escape(this.model.get('context')));
        return contextField.length - 65535;
    },

    substringContextFieldLength: function(){
        var contextList = this.getContextFieldData(),
            trimmedContextList = [],
            sortedContextList = [],
            whiteListedKeyNames = [],
            newContextList = [];

        sortedContextList = _.sortBy(contextList, function(context){ return context.label.length; });

        _.each(sortedContextList, function(context){
            var tempArgumentsJson = '';
            trimmedContextList.push(context);

            try{
                tempArgumentsJson = btoa(escape(JSON.stringify(trimmedContextList, null)));
                if(tempArgumentsJson.length <= 65535){
                    whiteListedKeyNames.push(context.name);
                }else{
                    return;
                }
            }catch(err){
                return;
            }
        }, this);


        if(!_.isEmpty(whiteListedKeyNames)){
            _.each(contextList, function(context, key){
                if(!_.isEmpty(context) && _.indexOf(whiteListedKeyNames, context.name) !== -1){
                    newContextList.push(context);
                }
            });
        }

        try{
            this.model.set('context', JSON.stringify(newContextList, null));
        }catch(err){
            this.model.set('context', '[]');
        }
    },

    getFieldsMeta: function(){
        var contextList = this.getContextFieldData(),
            fieldsMeta = [];

        _.each(contextList, function(contextItem){
            fieldsMeta.push({
                'name': 'attribute_label_' + contextItem.key
            });
        }, this);

        return fieldsMeta;
    }

})
