({
    extendsFrom: 'CreateView',

    initialize: function(options){
        this._super("initialize", [options]);
    },

    save: function() {
        if(!_.isUndefined(this.getField('context')))
            this.getField('context').beforeHandleSave();

        if(!_.isUndefined(this.getField('html_body')))
            this.model.set('html_body', this.getField('html_body')._getHtmlEditableField().val());

        this.checkContextFieldLength();
    },

    checkContextFieldLength: function(){
        if(!this.getField('context').isFieldLengthValid()){

            var symbolsAmountViolation = this.getField('context').getSymbolsAmountViolation(),
                errorMsg = app.lang.get('LBL_CONTEXT_MAX_LENGTH_EXCEEDED', 'PX_FlowchartTemplates').replace('%d', symbolsAmountViolation);

            app.alert.show('lb_tpl_context_max_length_exceeded', {
                level: 'confirmation',
                messages: errorMsg,
                onConfirm: _.bind(function(){
                    this.getField('context').substringContextFieldLength();
                    this.checkTemplateParseErrors();
                }, this)
            });
        }else{
            this.checkTemplateParseErrors();
        }
    },

    checkTemplateParseErrors: function(){
        var self = this,
            url = app.api.buildURL('lbTemplate/validateTemplate', '', {}, {}),
            engineTypesList = app.lang.getAppListStrings('px_lbtemplates_tpl_engine_type'),
            engineTypeName = self.model.get('tpl_engine_type'),
            engineTypeLabel = engineTypesList[engineTypeName] || '',
            errorMsg = app.lang
                .get('MSG_TEMPLATE_SYNTAX_ERROR_TEXT', self.module)
                .replace('$TPL_ENGINE$', engineTypeLabel);

        app.alert.show('validation-lb-templates', {level: 'process'});
        app.api.call('create', url,
            {
                name: this.model.get('name'),
                tpl_engine_type: this.model.get('tpl_engine_type'),
                context: this.model.get('context'),
                html_body: this.model.get('html_body'),
            },
            {
                success: _.bind(function (response) {
                    app.alert.dismiss('validation-lb-templates');
                    app.alert.dismiss('lb-templates-parse-error');
                    if(response.errorCode){
                        errorMsg =  errorMsg + '<br>' + response.errorText;
                        // errorMsg =  errorMsg + '<br>' + app.lang.get('MSG_TEMPLATE_SYNTAX_ERROR_WARNING', self.module);
                        app.alert.show('lb_template_err_variable', {
                            level: 'confirmation',
                            messages: errorMsg,
                            onConfirm: function(){
                                self.saveAndClose();
                            }
                        });
                    }else{
                        self.saveAndClose();
                    }
                    return;
                }),
                error: _.bind(function () {
                    app.alert.dismiss('validation-lb-templates');
                    app.alert.show('lb_template_err_variable', {
                        level: 'confirmation',
                        messages: errorMsg,
                        onConfirm: function(){
                            self.saveAndClose();
                        }
                    });
                    return;
                })
            });
    },

    hasUnsavedChanges: function() {
        var defaults,
            nonDefaultedAttributesChanged,
            defaultedAttributesChanged;

        if (this.resavingAfterMetadataSync) {
            return false;
        }

        defaults = this.model.getDefault() || {};

        var modelAttributes = {};

        _.each(this.model.attributes, function(value, fieldName){
            if(fieldName.match(/^attribute_\d+/ig)){

            }else if(value !== '' || !_.isUndefined(defaults[fieldName])){
                modelAttributes[fieldName] = value;
            }

        },this);

        nonDefaultedAttributesChanged = !_.isEqual(_.keys(defaults), _.keys(modelAttributes));
        defaultedAttributesChanged = !_.isEmpty(this.model.changedAttributes(defaults));

        return (this.model.isNew() && (nonDefaultedAttributesChanged || defaultedAttributesChanged));
    },

})
